/*
 * Copyright (C) 2011 Andrey Filkov
 *
 * This software is provided 'as-is', without any express or implied
 * warranty.  In no event will the authors be held liable for any damages
 * arising from the use of this software.
 *
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 *
 * 1. The origin of this software must not be misrepresented; you must not
 *    claim that you wrote the original software. If you use this software
 *    in a product, an acknowledgment in the product documentation would be
 *    appreciated but is not required.
 * 2. Altered source versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.
 * 3. This notice may not be removed or altered from any source distribution.
 *
 * File:    FConfigReader_r.cpp
 * Created: October 15, 2011
 */

#include "FConfigReader_r.h"
#include <pthread.h>


FConfigReader_r::~FConfigReader_r()
{
	pthread_rwlock_destroy(&m_lock);
}

FConfigReader_r* FConfigReader_r::CreateInstance(const std::string& file, const char com, const char delim, const char open, const char close)
{
	FConfigReader_r* p = new FConfigReader_r();
	if(p == 0)
		return 0;
	p->Init(file, com, delim, open, close);
	pthread_rwlockattr_t rwattr;
	if(pthread_rwlockattr_init(&rwattr) != 0)
	{
		delete p;
		return 0;
	}
	if(pthread_rwlockattr_setpshared(&rwattr, PTHREAD_PROCESS_SHARED) != 0)
	{
		delete p;
		return 0;
	}
	if(pthread_rwlock_init(&p->m_lock, &rwattr) != 0)
	{
		delete p;
		return 0;
	}
	return p;
}

bool FConfigReader_r::ReadConfig()
{
	bool ret = false;
	pthread_rwlock_wrlock(&m_lock);
	ret = FConfigReader::ReadConfig();
	pthread_rwlock_unlock(&m_lock);
}

void FConfigReader_r::GetSectionNames(std::vector<std::string>& sect)
{
	pthread_rwlock_rdlock(&m_lock);
	FConfigReader::GetSectionNames(sect);
	pthread_rwlock_unlock(&m_lock);
}

void FConfigReader_r::GetSectionData(std::map<std::string,std::string>& keyVal, const std::string& sect)
{
	pthread_rwlock_rdlock(&m_lock);
	FConfigReader::GetSectionData(keyVal, sect);
	pthread_rwlock_unlock(&m_lock);
}

bool FConfigReader_r::CheckKeys(std::vector<std::string>& keys, const std::string& sect)
{
	bool ret = false;
	pthread_rwlock_rdlock(&m_lock);
	ret = FConfigReader::CheckKeys(keys, sect);
	pthread_rwlock_unlock(&m_lock);
}
